#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "external.h"


int initdata(FILE *fp1, int *test, double *cut_off, double *maxdist){


	/* Check rate for termination */

	if(fscanf(fp1,"%d",test) != 1){
	  fprintf(stderr, "Error reading test rate from ga_fit.inp \n");
	  exit(1);
	}

	/* Cut-off distance for equivalences */

	if(fscanf(fp1,"%lf",cut_off) != 1){
	  fprintf(stderr, "Error reading cut-off distance for equivalences from ga_fit.inp \n");
	  exit(1);
	}

	*maxdist = SQR(*cut_off);

	if(fscanf(fp1,"%d",&popsize) != 1){
	  fprintf(stderr, "Error reading population size from ga_fit.inp \n");
	  exit(1);
	}

	if((popsize>MAXPOP)||(popsize<1)){
	  fprintf(stderr,"Error in population size \n");
	  exit(1);
	}

	if(fscanf(fp1,"%d",&maxgen) != 1){
	  fprintf(stderr, "Error reading maximum # of generations from ga_fit.inp \n");
	  exit(1);
	}

	if(fscanf(fp1,"%f",&pcross) != 1){
	  fprintf(stderr, "Error reading crossover probability from ga_fit.inp \n");
	  exit(1);
	}

	if(fscanf(fp1,"%f",&pmutation) != 1){
	  fprintf(stderr, "Error reading mutation probability from ga_fit.inp \n");
	  exit(1);
	}


	return 0;
}

int initpop(int lchrom, double sqr_cutoff, int list1[MAX_SEQ], int nlist1,
	    int list2[MAX_SEQ], int nlist2){

	int i,j;


	for(i=0;i < popsize ; i++){

		for(j=0;j < lchrom;j++){
		   
		   oldpop[i].chrom[j]=flip(0.5);
		}
		oldpop[i].fitness=objfunc(oldpop[i].chrom,ca,list1,nlist1,
					  list2,nlist2,sqr_cutoff);
		oldpop[i].parent1=0;
		oldpop[i].parent2=0;
		oldpop[i].xsite=0;

	}
    return 0;
}

int initialize(double sqr_cutoff, int list1[MAX_SEQ], int nlist1,
	       int list2[MAX_SEQ], int nlist2){

	int lchrom = MAXSTRING;
	time_t t;

	nmutation=0;
	ncross=0;
	bestfit.no_bestca=0;
	bestfit.fitness=0.0;
	bestfit.generation=0;

	/* initialize random number generator */

        srand((unsigned) time(&t));

	if((initpop(lchrom,sqr_cutoff,list1,nlist1,list2,nlist2))!=0){
	  fprintf(stderr,"Error in generating a random population for GA\n");
	  exit(1);
	}

	statistics(popsize,&max,&avg,&min,&sumfitness,oldpop,lchrom);

	return (lchrom);
}





