#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define VER	"Alex C.W.May and Mark S.Johnson 1994"

#define MAX_MAT 15	/* Limit to matrices in matrices.h */

#define MAX_SEQ 50	/* Limit to the number of sequences to be aligned */

#define MAX_LEN 1002	/* Limit to the length of sequences = MAX_LEN-52 */

#define TRUE	0

#define FALSE	1

#define NUMBER_ACID 20

#define MAXPOP 100

#define MAXSTRING 56

#define BIT_POSITIONS 8

#define RANDOM (rand()/(RAND_MAX+1.0))

#define SQR(x) ((x)*(x))

#define MAXGAP 20.0

#define MISMATCH 0.0

#define MAX_DIST 99999.0

#define PI 3.141592654

#define MIN(A,B) ((A) < (B)? (A) : (B))

#define MAX(A,B) ((A) > (B)? (A) : (B))

#define INPUT "ga_fit.inp"


/******** Structure for superposition ********/

struct atom_rec{
	int atom_number;
	char atom_name[4];
	char residue_name[4];
	char chain;
	char insertion_code;
	char residue_number[5];
	float xyz[3];
};

/******** Structures for GA ********/

struct individual{

	int chrom[MAXSTRING];
	double fitness;
	int parent1;
	int parent2;
	int xsite;
};

struct bestever{

	int chrom[MAXSTRING];
	int no_bestca;
	double fitness;
	int generation;
};

/******** Function prototypes for superposition ********/

int read_Brookhaven_COMPND_record( char *buffer );

int read_Brookhaven_ATOM_record( char *buffer );

char three_let_to_1let(char threelett[3]);

int matrix_mult(float y[3],double x[3][3],float z[3]);

double distance(float a[3],float b[3]);

int translate(float a[3],float b[3]);

int max_len(int list[MAX_SEQ], int nlist, int seq_len[MAX_SEQ]);

float max_dim(int list[MAX_SEQ], int nlist, float largest_dim[MAX_SEQ]);

double **dmatrix(int nrl,int nrh,int ncl,int nch);

void free_dmatrix(double **m,int nrl,int nrh,int ncl,int nch);

int **imatrix(int nrl,int nrh,int ncl,int nch);

void free_imatrix(int **m,int nrl,int nrh,int ncl,int nch);

float *vector(int nl,int nh);

float **fmatrix(int nrl,int nrh,int ncl,int nch);

void free_vector(float *v,int nl,int nh);

void free_fmatrix(float **m,int nrl,int nrh,int ncl,int nch);

int *ivector(int nl,int nh);

void free_ivector(int *v,int nl,int nh);

void jacobi(float **a,int n,float d[],float **v,int *nrot);

void eigsrt(float d[],float **v,int n);

void nrerror(char error_text[]);

int bits2int(int *string,int len);

double fast_pair_um(int no_res1,int no_res2,double **mat2,double gap_pen,
		    double sqr_cutoff);

double ga_pair_um(int no_res1,int no_res2,double **mat2,int *len,
		  double gap_pen);

void ga_gap0_trace(double gap_pen);

void ga_gap2_trace(double gap_pen);

void ga_gap1_trace(double gap_pen);

int print_align_ca(int no_res1, int no_res2, int list1[MAX_SEQ], int nlist1,
		   int list2[MAX_SEQ], int nlist2, double **mat1, int len,
		   double sqr_cutoff,int *end_no_bestca);

double objfunc(int *string,float ca[MAX_SEQ][(MAX_LEN-52)][3],
	       int list1[MAX_SEQ],int nlist1,int list2[MAX_SEQ],
	       int nlist2,double sqr_cutoff);

int print_objfunc(int *string,float ca[MAX_SEQ][(MAX_LEN-52)][3],
		  int list1[MAX_SEQ],int nlist1,int list2[MAX_SEQ],
		  int nlist2,double sqr_cutoff,double *score1,
		  double *score2,int *align_len,int *end_no_equiv,
		  int mat,int pen);

int update_align_ca(int no_res1,int no_res2,double **mat1,int len,
		    double sqr_cutoff,int *no_equiv,double *rmsd,
		    double gap_pen);

int comp_equiv(int no_equiv1,int equiv1[2][(MAX_LEN-52)],int no_equiv2,int equiv2[2][(MAX_LEN-52)]);

int write_um (int no_struc);

int pairwise_comp(int prot1,int prot2,double sqr_cutoff,int *no_equiv,
		  double *rmsd,double *pfte,int *all_equiv);

double calc_structural_dist_metric(int prot1,int prot2,double cut_off,
				   double rmsd,double pfte);

int print_common_equiv(int t_len,
		       int no_struc,
		       const int *all_equiv);

int common_equiv(int no_struc, const int *all_equiv, int **mat);

int common_pairwise_comp(int prot1, int prot2, const int *all_equiv,
			 const int **mat,
			 int t_len, int no_struc, int common_no_equiv,
			 double *rmsd);

/******** Function prototypes for GA ********/

int ga_fit(int list1[MAX_SEQ],int nlist1,int list2[MAX_SEQ],
	   int nlist2,double *score_a,int *end_no_bestca,
	   int mat,int pen,int *t_len);

int generation(int lchrom,double sqr_cutoff,int list1[MAX_SEQ], int nlist1,
	       int list2[MAX_SEQ], int nlist2);

int flip(float probability);

int rnd(int low,int high);

int select(int popsize,double *sumfitness,struct individual pop[MAXPOP]);

int mutation(int alleleval,float pmutation,int *nmutation);

int crossover(int *parent1,int *parent2,int *child1,int *child2,
	      int lchrom,int *ncross,int *nmutation,int *jcross,
	      float pcross,float pmutation);

int initdata(FILE *fp1, int *test, double *cut_off, double *maxdist);

int initpop(int lchrom, double sqr_cutoff, int list1[MAX_SEQ], int nlist1,
	    int list2[MAX_SEQ], int nlist2);

int initialize(double sqr_cutoff, int list1[MAX_SEQ], int nlist1,
	       int list2[MAX_SEQ], int nlist2);

int statistics(int popsize,double *max,double *avg,double *min,
	       double *sumfitness,struct individual pop[MAXPOP],
	       int lchrom);

int igray(int n,int is);

/******** Global declarations for GA ********/

int chromosome[MAXSTRING];

struct individual oldpop[MAXPOP];

struct individual newpop[MAXPOP];

struct bestever bestfit;

int popsize;

int lchrom;

int gen;

int maxgen;

float pcross;

float pmutation;

double sumfitness;

int nmutation;

int ncross;

double avg;

double max;

double min;

int jcross;

int i;

/******** Global declarations for superposition ********/

float start_ca[MAX_SEQ][(MAX_LEN-52)][3];

float ca[MAX_SEQ][(MAX_LEN-52)][3];

char residue_name[MAX_SEQ][(MAX_LEN-52)][4];

char residue_number[MAX_SEQ][(MAX_LEN-52)][5];

int align [MAX_SEQ][(MAX_LEN-52)];

double **gap2;

double **gap1;

double **gap0;

int c;

int j,k;

