#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "external.h"

/* Function to translate 3 letter AA code to 1 letter AA code */

char three_let_to_1let(char threelett[3]){

int i;  /* counter */

char single[NUMBER_ACID]  = {
                   'A',  'C',  'D',  'E',  'F',  'G',  'H',  'I',
		   'K',  'L',  'M',  'N',  'P',  'Q',  'R',  'S',
		   'T',  'V',  'W',  'Y'};

char *three_letter[NUMBER_ACID] = {
                   "ALA","CYS","ASP","GLU","PHE","GLY","HIS","ILE",
		   "LYS","LEU","MET","ASN","PRO","GLN","ARG","SER",
		   "THR","VAL","TRP","TYR"};


for(i=0;i<3;i++){

    threelett[i]=toupper(threelett[i]);
}

   for(i=0;i<NUMBER_ACID;i++){

       if(strcmp(threelett,three_letter[i])==0){

	 return single[i];
       }

   }

/* Failure */

return '\0';
}


/* Function to multiply 3x3 matrix by 3x1 vector */

int matrix_mult(float y[3],double x[3][3],float z[3]){

int m,n;


	for(m=0;m<3;m++){

		z[m]=0.0;
	
		for(n=0;n<3;n++){

		   z[m] +=x[m][n]* y[n];

                }

		
	}

return 0;
}

/* Function to return the squared straight line distance between any two

points in Cartesian space */


double distance(float a[3],float b[3]){

int i;
double answer=0.0;

	for(i=0;i<3;i++)
	    answer+=SQR(b[i]-a[i]);


return (answer);
}

/* Function to translate position by 3x1 vector */


int translate(float a[3],float b[3]){

int i;

       for(i=0;i<3;i++)	
	   a[i] += b[i];

return 0;
}

/* Function to determine longest sequence of list to align as set */

int max_len(int list[MAX_SEQ], int nlist, int seq_len[MAX_SEQ]){

int max;

int i,j;

for(i=0; i<nlist; i++){

   if(i==0){

     max = seq_len[list[i]]; /* initialise max with first seq */
     j = i;
   }
   else if(seq_len[list[i]] > max){

	  max = seq_len[list[i]];
	  j = i;
   }
}

return j; /* array index in list for max */
}


/* Function to determine average longest dimension of list to align as set */

float max_dim(int list[MAX_SEQ], int nlist, float largest_dim[MAX_SEQ]){

float total;
int i;


total = 0.0;

for(i=0; i<nlist; i++)
   total += largest_dim[list[i]];


return (total/(float)nlist);
}


/******** Numerical Recipes routines ********/

double **dmatrix(int nrl,int nrh,int ncl,int nch){

	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in dmatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
		m[i] -= ncl;
	}
	return m;
}

void free_dmatrix(double **m,int nrl,int nrh,int ncl,int nch){

	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

int **imatrix(int nrl,int nrh,int ncl,int nch)
{
	int i;
	int **m;

	m=(int **) malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
	if (!m) nrerror("allocation failure 1 in imatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(int *) malloc((unsigned) (nch-ncl+1)*sizeof(int));
		if (!m[i]) nrerror("allocation failure 2 in imatrix()");
		m[i] -= ncl;
	}
	return m;
}

void free_imatrix(int **m,int nrl,int nrh,int ncl,int nch)

{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

float **fmatrix(int nrl,int nrh,int ncl,int nch)

{
	int i;
	float **m;

	m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure 1 in fmatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
		if (!m[i]) nrerror("allocation failure 2 in fmatrix()");
		m[i] -= ncl;
	}
	return m;
}

void free_fmatrix(float **m,int nrl,int nrh,int ncl,int nch)

{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

float *vector(int nl,int nh)

{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

void free_vector(float *v,int nl,int nh)

{
	free((char*) (v+nl));
}

int *ivector(int nl,int nh)

{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

void free_ivector(int *v,int nl,int nh)

{
	free((char*) (v+nl));
}

void nrerror(char error_text[]){

	void exit();

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}


int igray(int n,int is){

int ish;
int ans,idiv;


	if(is >= 0)
	  return n ^ (n >> 1);

	ish=1;
	ans=n;

	for(;;){

	   ans ^= (idiv=ans >> ish);

	   if(idiv <= 1 || ish == 16) return ans;

	   ish <<= 1;
	}
}


	




