/*

   To define pairwise equivalences and rmsd in multipe superposition

   To calculate pairwise fractional topological equivalence (PFTE) according
   to Johnson et al. (1990) J.Mol.Evol. 30: 43-59.


   Alex May

   Last revision:	03 August 1994
			11 October 1994

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "external.h"

#define MAX_CDE 25

extern char code[MAX_SEQ][MAX_CDE];
extern char seq[MAX_SEQ][MAX_LEN];
extern int len[MAX_SEQ];
extern int ren[MAX_SEQ];

int pairwise_comp(int prot1,int prot2,double sqr_cutoff,int *no_equiv,
		  double *rmsd,double *pfte,int *all_equiv){

register int i,j,k;
register int no_match;
double d;
double total_dist;
register int smaller_structure;


if(len[prot1] != len[prot2]){
  fprintf(stderr,"Error0 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}

j=0;
k=0;
no_match=0;
*no_equiv=0;
total_dist=0.0;

for(i=0;i<len[prot1];i++){

   if(seq[prot1][i+1] != '-')
     j++;

   if(j > ren[prot1]){
     fprintf(stderr,"Error1 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
     exit(1);
   }

   if(seq[prot2][i+1] != '-')
     k++;

   if(k > ren[prot2]){
     fprintf(stderr,"Error2 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
     exit(1);
   }

   if((seq[prot1][i+1] != '-') && (seq[prot2][i+1] != '-')){

     no_match++;

     d=distance(start_ca[prot1][j-1],start_ca[prot2][k-1]);

     if(d < sqr_cutoff){

       if(*no_equiv > no_match){
	 fprintf(stderr,"Error3 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
	 exit(1);
       }

       total_dist += d;

       (*no_equiv)++; /* pairwise equivalence */

       all_equiv[i]++;
     }
   }
}

if(total_dist>MAX_DIST){
  fprintf(stderr,"Error4 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}
else{

 if((*no_equiv > 0)&&(total_dist >= 0.0)){

   *rmsd = sqrt((total_dist/(double)*no_equiv));

   smaller_structure = MIN(ren[prot1],ren[prot2]);

   if(smaller_structure > 0)
     *pfte = (double)*no_equiv/(double)smaller_structure;

   else{
     fprintf(stderr,"Error5 in pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
     exit(1);
   }
 }

 else{
   *no_equiv=(-99);
   *rmsd=(-99.0);
   *pfte=(-99.0);
 }
}

return 0;
}

/*

   To define pairwise rmsds over common (all pairwise) equivalences


   Alex May

   Last revision:       13 October 1994


*/

int common_pairwise_comp(int prot1, int prot2, const int *all_equiv,
			 const int **mat,
			 int t_len, int no_struc, int common_no_equiv,
			 double *rmsd){

register int threshold;
double d;
double total_dist;
register int no_equiv;

register int i;


threshold = (SQR(no_struc) - no_struc)/2.0;

total_dist = 0.0;
no_equiv = 0;

for(i=0;i<t_len;i++){

   if(all_equiv[i] == threshold){

     d=distance(start_ca[prot1][mat[prot1][i]],start_ca[prot2][mat[prot2][i]]);

     total_dist += d;
     no_equiv++;
   }
}

if(no_equiv != common_no_equiv){
  fprintf(stderr,"Error0 in common_pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}

if(total_dist>MAX_DIST){
  fprintf(stderr,"Error1 in common_pairwise_comp():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}

else{

 if((common_no_equiv > 0)&&(total_dist >= 0.0))
  *rmsd = sqrt((total_dist/(double)common_no_equiv));

 else
  *rmsd=(-99.0);
}

return 0;
}