/*

   To calculate pairwise structural distance metric according to
   Johnson et al. (1990) J.Mol.Evol. 30: 43-59.


   Alex May

   Last revision:	03 August 1994


*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "external.h"

#define MAX_CDE 25

extern char code[MAX_SEQ][MAX_CDE];

double calc_structural_dist_metric(int prot1,int prot2,double cut_off,
				   double rmsd,double pfte){

double srms;
double w1, w2;
double sdist;

if((rmsd >= 0.0) && (cut_off > 0.0))
  srms = 1.0 - (rmsd/cut_off);

else{
  fprintf(stderr,"Error0 in calc_structural_dist_metric():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}

if(pfte > 0.0)
  w1 = 0.5 * ((1.0 - pfte) + (1.0 - srms));

else{
  fprintf(stderr,"Error1 in calc_structural_dist_metric():\t%s and %s\n",code[prot1],code[prot2]);
  exit(1);
}

w2 = 0.5 * (pfte + srms);

sdist = -100.0 * (log((w1 * pfte) + (w2 * srms)));

return (sdist);
}
